static char rcsid[] = "@(#)$Id: write_hdr.c,v 2.0 2007/08/25 11:51:45 hurtta Exp $";

/******************************************************************************
 *  The Elm (ME+) Mail System  -  $Revision: 2.0 $   $State: Exp $
 *
 *  Author: Kari Hurtta <hurtta+elm@posti.FMI.FI> 
 *      or  Kari Hurtta <elm@elmme-mailer.org>
 *****************************************************************************/


#include "def_addr.h"

DEBUG_VAR(Debug,__FILE__,"header");

#define LEN (out_state_ftell(mailer)-POS)
#define WRAP print_EOLN(mailer,top_encoding), \
 (POS = out_state_ftell(mailer)), state_putc(' ',mailer)

#define PUT(c)     {  int c1=(c); state_putc(c1,mailer); \
                      DPRINT(Debug,9, (&Debug,"%c",c1)); }
#define PUTS(s)    {  const char * s1=(s); state_puts(s1,mailer); \
                      DPRINT(Debug,9, (&Debug,"%s",s1)); }

static void print_phrase P_((out_state_t *mailer, const struct string *phrase,
			     int idx, long *posp, int top_encoding, 
			     int enmime, charset_t hdr_charset));

static void print_phrase(mailer,phrase,idx,posp,top_encoding,enmime,hdr_charset)
     out_state_t *mailer;
     const struct string *phrase;
     int idx;
     long *posp;
     int top_encoding;
     int enmime;
     charset_t hdr_charset;
{
    long POS = *posp;
    char * c;                    
    char * buffer = string_to_hdr(HDR_PHRASE, phrase,         
				  hdr_charset, enmime, NULL); 

    DPRINT(Debug,9, (&Debug,"{phrase quoted=%s}\n", buffer)); 
    DPRINT(Debug,9, (&Debug, "\n          [%d] ",idx));       

    for (c = buffer; *c != '\0'; c++) {                       
	
	if (*c == '\n' || (whitespace(*c) && (LEN > 120))) {    
	    WRAP;                                               
	    DPRINT(Debug,9,(&Debug, "[WRAP]\n          [%d] ", idx)); 
	
	} else {                                                
	    PUT(*c);                                            
	}                                                       
    }
    free(buffer);                                             
    
    *posp = POS;
}

#define PRINT_PHRASE(phrase) print_phrase(mailer,phrase,idx,&POS,\
  top_encoding,enmime,hdr_charset)

void write_addr_header(mailer,hdr_name,addr,top_encoding,enmime,hdr_charset,add_group)
     out_state_t *mailer;
     char *hdr_name;
     struct addr_list * addr;
     int top_encoding;
     int enmime;
     charset_t hdr_charset;
     int add_group;
{
    long POS = out_state_ftell(mailer);

    int  glen = addr_list_group_count(addr);
    int len   = addr_list_item_count(addr);
    int idx;
    int gidx  = -1;
    int last_group = -1;



    state_puts(hdr_name,mailer);
    state_puts(": ",mailer);

    DPRINT(Debug,9,
	   (&Debug, "write_addr_header: (enmime=%d) %s:",
	    enmime,hdr_name));
    
    for (idx = 0; idx < len; idx++) {

	int group = -1;
	const struct address * address = 
	    addr_list_get_item(addr,idx,&group);
	
	const char          * ad          = address_get_ascii_addr(address);
	const struct string * fullname    = address_get_phrase(address);
	const struct string * comment     = address_get_comment(address);

	const int al = ad        ? strlen(ad )          : 0;
	const int fn = fullname  ? string_len(fullname)  : 0;
	const int cm = comment   ? string_len(comment)   : 0;

	DPRINT(Debug,9,
	       (&Debug, "\n          [%d] ",idx));

    next_phrase:
	if (last_group != group &&
	    last_group != -1) {
	    PUT(';');
	    last_group = -1;	
	}
	    
	if (!ad) {
	    continue;
	}

	if (idx > 0) {
	    PUT(',');
	    if (LEN > 70 || al + fn + cm + LEN > 76) {
		WRAP;
		DPRINT(Debug,9,
		       (&Debug,"[WRAP]\n          [%d] ",idx));
	    } else {
		PUT(' ');
	    }
	}
	    
	if (gidx < group &&  
	    group >= 0 && last_group == -1) {

	    /* Print empty groups */

	    const struct string * groupname =
		addr_list_get_group(addr,++gidx);

	    PRINT_PHRASE(groupname);	    
	    PUT(':');
	    last_group = gidx;

	    if (gidx < group)
		goto next_phrase;

	    PUT(' ');
	}

	if (group >= 0 && last_group == -1) {
	    const struct string * groupname =
		addr_list_get_group(addr,group);

	    PRINT_PHRASE(groupname);	    
	    PUTS(": ");
	    last_group = group;
	}


	if (fn || ! ad[0] || '@' == ad[0]) {
	    if (fn) {
		PRINT_PHRASE(fullname);
	    }

	    if (LEN > 120 || (LEN > 70 && fn > 70)) {
		WRAP;
		DPRINT(Debug,9,
		       (&Debug, "[WRAP]\n          [%d] ",idx));
	    } else {
		PUT(' ');

	    }

	    PUT('<');
	    PUTS(ad);
	    PUT('>');
	} else {
	    PUTS(ad);
	}

	if (cm) {
	    char * buffer = string_to_hdr(HDR_COMMENT, comment,
					  hdr_charset, enmime,NULL);
	    char * c;
	    
	    DPRINT(Debug,9,
		   (&Debug,"\n          [%d] ",idx));

	    if (LEN > 85) {
		DPRINT(Debug,9,
		       (&Debug, "[WRAP]\n          [%d] ",idx));
		WRAP;
	    } else {
		state_putc(' ',mailer);
		DPRINT(Debug,9,
		       (&Debug ," "));
	    }
   
	    DPRINT(Debug,9,
		   (&Debug, "\n          {comment encoded=%s}",buffer));
	    DPRINT(Debug,9,
		   (&Debug, "\n          [%d] ",idx));

	    PUT('(');
	    for (c = buffer; *c != '\0'; c++) {
		if (*c == '\n' || 
		    (whitespace(*c) && (LEN > 100))) {
		    WRAP;
		    DPRINT(Debug,9,
			   (&Debug, "[WRAP]\n          [%d] ",idx));
		} else {
		    PUT(*c);
		}
	    }
	    PUT(')');
	    free(buffer);
	}
    }


    if (last_group != -1) {
	PUT(';');
	last_group = -1;	
    }

    if (gidx < glen && idx > 0) {
	PUT(',');
	if (LEN > 70) {
	    WRAP;
	    DPRINT(Debug,9,
		   (&Debug,"[WRAP]\n          [%d] ",idx));
	} else {
	    PUT(' ');
	}
    }

    while (gidx+1 < glen) {
	const struct string * groupname =
	    addr_list_get_group(addr,++gidx);
	
	
	PRINT_PHRASE(groupname);	    
	PUTS(":;");
	last_group = -1;	

	if (gidx < glen) {
	    PUT(',');
	    if (LEN > 70) {
		WRAP;
		DPRINT(Debug,9,
		       (&Debug,"[WRAP]\n          [%d] ",idx));
	    } else {
		PUT(' ');
	    }
	}
    }

    print_EOLN(mailer,top_encoding);
    DPRINT(Debug,9,
	   (&Debug, "[EOLN]\n"));
}


/*
 * Local Variables:
 *  mode:c
 *  c-basic-offset:4
 *  buffer-file-coding-system: iso-8859-1
 * End:
 */
