static char rcsid[] = "@(#)$Id: message_screen.c,v 2.0 2007/08/25 11:52:08 hurtta Exp $";

/******************************************************************************
 *  The Elm (ME+) Mail System  -  $Revision: 2.0 $   $State: Exp $
 *
 *  Author: Kari Hurtta <hurtta+elm@posti.FMI.FI> 
 *      or  Kari Hurtta <elm@elmme-mailer.org>
 ******************************************************************************
 * Inludes  code from Elm 2.4 src/screen.c which have following copyright:
 *
 *  The Elm Mail System 
 *
 *			Copyright (c) 1988-1992 USENET Community Trust
 *			Copyright (c) 1986,1987 Dave Taylor
******************************************************************************
 *  Incorparated Elm 2.5 code from src/screen.c. 
 *  That code was following copyright:
 *
 *  The Elm Mail System
 *
 *                      Copyright (c) 1988-1995 USENET Community Trust
 *                      Copyright (c) 1986,1987 Dave Taylor
 *****************************************************************************/

#include "def_mcommon.h"
#include "s_elm.h"

static void print_dummy_header_line P_((int message_number,int selected,
					struct menu_context  *page));
static void print_header_line P_((struct header_rec *entry, int message_number,
				  int selected, struct string *from,
				  int really_to, struct menu_context  *page,
				  char *status, int tagged));

void mc_screen_print_mbx_line(u,idx, page)
     union mcommon_union *u;
     int idx;
     struct menu_context  *page;
{
    struct MailboxView *mbxview = u->mbx.mw;
    
    /* from function show_headers() */
    int current = get_current(mbxview);
    struct header_rec * hdr = give_header(mbxview,idx);

    if (hdr) {
	char * status = show_status_message(mbxview,idx);
	struct string * newfrom = NULL;
	int using_to = DisplayAddress(hdr,&newfrom);
	
	int tagged = ' ';

	if (ison_status_message(mbxview,idx,status_basic,TAGGED))
	    tagged = '+';
	else if (ison_status_message(mbxview,idx,status_1,S1_FLAGGED))
	    tagged = 'F';
	else
	    tagged = ' ';


	if (idx == current-1) 
	    print_header_line(hdr, idx+1, TRUE, newfrom, using_to, page,
			      status,tagged);
	else
	    print_header_line(hdr, idx+1, FALSE, newfrom, using_to, page,
			      status,tagged);
	free_string(&newfrom);
    } else {		
	if (idx == current-1) 
	    print_dummy_header_line(idx+1,TRUE, page);
	else
	    print_dummy_header_line(idx+1,FALSE, page);
    }    
}


static void print_dummy_header_line(message_number,selected,page)
     int message_number;
     int selected;
     struct menu_context  *page;
{

    int l;
    int X;
    struct string * buffer = NULL;
    struct string * buffer1 = NULL;
    int LINES, COLUMNS;

    menu_get_sizes(page, &LINES, &COLUMNS);

    if (selected && has_highlighting && ! arrow_cursor) 
	menu_StartXX(page,pg_INVERSE);

    buffer = format_string(CATGETS(elm_msg_cat, ElmSet, 
				   ElmNotCollected,
				   "%.2s-- %-3d  <--- message not collected --->"),
			   (selected && arrow_cursor)? "->" : "  ",
			   message_number);
		
    X = 0;

    buffer1 = curses_printable_clip(buffer,&X,COLUMNS,
				    &l,
				    COLUMNS);
    
    free_string(&buffer);

    if (buffer1) {
	menu_Write_to_screen(page,FRM("%S"),buffer1);
	free_string(&buffer1);
    }

    if (selected && !arrow_cursor) {
	while (l < COLUMNS) {
	    menu_Writechar(page,' ');
	    l++;
	}
    }
        
    if (selected && has_highlighting && ! arrow_cursor) 
	menu_EndXX(page,pg_INVERSE);

    menu_Writechar(page,'\r');
    menu_Writechar(page,'\n');
}

static struct string * mlist_token P_((enum mlist_tokens idx));
static struct string * mlist_token(idx)
     enum mlist_tokens idx;
{
    const struct string * X = NULL;

    if (user_mlist_conf)
	X = get_mlist_token(user_mlist_conf,idx);

    if (!X && system_mlist_conf)
	X = get_mlist_token(system_mlist_conf,idx);

    if (X)
	return format_string(FRM(" %12S/"),X);
    

    switch (idx) {
    case mlist_to_me_token:  
	return format_string(FRM(" ------------/"));
    case mlist_to_many_token:
	return format_string(FRM(" ======------/"));
    case mlist_cc_me_token:
	return format_string(FRM(" ============/"));

    }

    return NULL;
}

#if ANSI_C
static mlist_match_id_f match_list_id;
#endif
static int match_list_id P_((char * list_id,
			     struct list_info *list));
static int match_list_id(list_id,list)
     char * list_id;
     struct list_info *list;
{
    const char * id = get_list_id(list);

    if (!id)
	return  0;
    
    return 0 == strcmp(list_id,id);
}

static void print_header_line(entry,message_number,selected,from,really_to,
			      page,status,tagged)
     struct header_rec *entry;
     int message_number; 
     int selected;
     struct string *from;
     int really_to;
     struct menu_context  *page;
     char * status;
     int tagged;
{
    /** Print in buffer the message header ... entry is the current
	message entry, 'from' is a modified (displayable) from line, 
	'highlight' is either TRUE or FALSE, and 'message_number'
	is the number of the message.
    **/
	
    int who_width = 18, subj_width = 0;
    int header_without_subj_width;

    struct string * buffer = NULL;
    struct string * buffer1 = NULL;
    struct string * bufferwho = NULL;
    struct string * list_buffer = NULL;
    int X,len1,fill_len;
    int was_len;
    int LINES, COLUMNS;


    enum list_mode {
	lm_unknown,
	lm_myself_to,
	lm_myself_cc,
	lm_list_id,
	lm_mlist
    } list_mode = lm_unknown;

    int other_seen = 0;  /* if other than sender or reader
			    seen on address headers 
			 */

    const struct mlist_info * mlist = NULL;

    if (!status)
	status = "??";

    /* Check is reader also on recipient headers */
    if (entry->list_info)
	list_mode = lm_list_id;
    

    if (entry->cc) {
	int i, len = addr_list_item_count(entry->cc);

	for (i = 0; i < len; i++) {
	    int g = -1;
	    const struct address * address = 
		addr_list_get_item(entry->cc,i,&g);

	    
	    if (!okay_address_l(address,NULL))
		list_mode = lm_myself_cc;
	    
	    if (okay_address_l(address,entry->from))
		other_seen++;
	}
    }


    if (entry->to) {
	int i, len = addr_list_item_count(entry->to);


	for (i = 0; i < len; i++) {
	    int g = -1;
	    const struct address * address = 
		addr_list_get_item(entry->to,i,&g);

	    if (!okay_address_l(address,NULL))
		list_mode = lm_myself_to;
	    
	    if (okay_address_l(address,entry->from))
		other_seen++;
	}
    }


    if (list_mode  == lm_unknown ||
	lm_list_id == list_mode) {

	struct mlist_scan * X1 = NULL;

	if (user_mlist_conf) 
	    X1 = mlist_get_matches(user_mlist_conf,entry,match_list_id);

	if (system_mlist_conf) {
	    struct mlist_scan * X2 = mlist_get_matches(system_mlist_conf,
						       entry,match_list_id);

	    if (X1) {
		append_mlist_scan(X1,X2);
		free_mlist_scan(&X2);
	    } else
		X1 = X2;
	}
	
	if (X1) {
	    mlist = get_mlist_info(X1);

	    if (mlist)
		list_mode = lm_mlist;

	    free_mlist_scan(&X1);
	}

    }


    if (show_mlists) {
	const int MLIST_LEN = 12;

	/* 1) Just to me or to sender only */

	if (lm_myself_to == list_mode &&
	    0 == other_seen)
	    list_buffer = mlist_token(mlist_to_me_token);

	
	/* 2) to me and others */

	else if (lm_myself_to == list_mode)
	    list_buffer = mlist_token(mlist_to_many_token);


	/* 3) cc to me */

	else if (lm_myself_cc == list_mode)
	    list_buffer = mlist_token(mlist_cc_me_token);

	/* 4) mailing list post */
	
	else if (lm_mlist == list_mode) {
	    const struct string * list_phrase = 
		get_mlist_display_name(mlist);

	    if (list_phrase) {
		int X = 0,len1;
		struct string * N = 
		    curses_printable_clip(list_phrase,
					  &X,MLIST_LEN,&len1,MLIST_LEN);

		int fill_len = MLIST_LEN - len1;

		list_buffer = format_string(FRM(" %S%*s/"),
					    N,fill_len,"");

		free_string(&N);

	    } else
		goto list_name_failed;

	} else if (lm_list_id == list_mode) {
	    const struct string * list_phrase = 
		get_list_phrase(entry->list_info);
	    const char * list_id = 
		get_list_id(entry->list_info);

	    
	    if (list_phrase) {
		int X = 0,len1;
		struct string * N = 
		    curses_printable_clip(list_phrase,
					  &X,MLIST_LEN,&len1,MLIST_LEN);

		int fill_len = MLIST_LEN - len1;

		list_buffer = format_string(FRM(" %S%*s/"),
					    N,fill_len,"");

		free_string(&N);

	    } else if (list_id) {
		/* FIXME?  DO not take account header charset */
		list_buffer = format_string(FRM(" %*.*s/"),
					    -MLIST_LEN,MLIST_LEN,
					    list_id);

	    } else 
		goto list_name_failed;

	   
	/* 5) Show To: -header */

	} else {
	list_name_failed:
	    if (really_to) {
		/* If we are idplaying anyway To -address, do not display
		   it twice 
		*/
	    
		list_buffer = new_string(display_charset);
		who_width += MLIST_LEN +2;

	    } else if (entry->to) {
	    
		int  len = addr_list_item_count(entry->to);

		if (1 == len) {
		    int g;
		    const struct address * address = 
			addr_list_get_item(entry->to,0,&g);
		    const char * addr =
			address_get_ascii_addr(address);
		    const struct string * fullname =
			address_get_phrase(address);

		    if (fullname &&
			string_len(fullname) > 0) {
			int X = 0,len1;
			struct string * N = 
			    curses_printable_clip(fullname,
						  &X,MLIST_LEN-2,&len1,MLIST_LEN-2);
			
			int fill_len = MLIST_LEN - len1 -2;
			
			list_buffer = format_string(FRM(" (%S%*s)/"),
						N,fill_len,"");
		    
			free_string(&N);
		    } else if (addr) {
		    
			/* FIXME?  DO not take account header charset */
			list_buffer = format_string(FRM(" (%*.*s)/"),
						    -MLIST_LEN+2,
						    MLIST_LEN-2,
						    addr);
		    } else
			goto to_failure;
		} else
		    goto to_failure;
	    }

	    /* 6) No To: -header or number of addresses != 1 */
	    else {
	    to_failure:

		list_buffer = format_string(FRM(" ***         /"));

	    }

	}

    } else {

	/* 1) Just to me or to sender only */

	if (lm_myself_to == list_mode &&
	    0 == other_seen)
	    list_buffer = format_string(FRM(" %c"),
					to_chars[0] ? 
					to_chars[0] :
					DEFAULT_TO_CHARS[0]);
	
	/* 2) to me and others */
	else if (lm_myself_to == list_mode)
	    list_buffer = format_string(FRM(" %c"),
					to_chars[1] ? 
					to_chars[1] :
					DEFAULT_TO_CHARS[1]);
	
	/* 3) cc to me */

	else if (lm_myself_cc == list_mode ) 
	    list_buffer = format_string(FRM(" %c"),
					to_chars[2] ? 
					to_chars[2] :
					DEFAULT_TO_CHARS[2]);


	/* 4) mailing list post */	
	else if (lm_list_id == list_mode ||
		 lm_mlist == list_mode)
	    list_buffer = format_string(FRM(" %c"),
					to_chars[4] ? 
					to_chars[4] :
					DEFAULT_TO_CHARS[4]);

	/* 5) unknown */	
	else
	    list_buffer = format_string(FRM(" %c"),
					to_chars[3] ? 
					to_chars[3] :
					DEFAULT_TO_CHARS[3]);
    }

    menu_get_sizes(page, &LINES, &COLUMNS);


    /* truncate 'from' to 18 characters -
     * this includes the leading "To" if really_to is true.
     */
	
    if (*entry->time_menu == '\0')
	make_menu_date(entry); 

    /* show "To " in a way that it can never be truncated. */
    if (really_to) {
	who_width -= 3;
    }

    if (selected && has_highlighting && ! arrow_cursor) 
	menu_StartXX(page,pg_INVERSE);

    X = 0;
    bufferwho = curses_printable_clip(from,&X,who_width,&len1,who_width);

    if (!bufferwho) {
	bufferwho = new_string(display_charset);
	fill_len = who_width;
    } else
	fill_len = who_width - len1;

    if (fill_len < 0)
	fill_len = 0;

    if (entry->lines < 0) {
        buffer = format_string(FRM("%.2s%.2s%c%-3d %.10s%S %.3s%S%*s (?)    "),
			       (selected && arrow_cursor)? "->" : "  ",
			       status,
			       tagged,
			       message_number,
			       entry->time_menu,
			       list_buffer,
			       (really_to ? "To " : ""),
			       /* give max and min width parameters for 'from' */
			       bufferwho,

			       fill_len,""
			       );
    } else
	buffer = format_string(FRM("%.2s%.2s%c%-3d %.10s%S %.3s%S%*s (%d) %s"),
			       (selected && arrow_cursor)? "->" : "  ",
			       status,
			       tagged,
			       message_number,
			       entry->time_menu,
			       list_buffer,
			       (really_to ? "To " : ""),
			       /* give max and min width parameters for 'from' */
			       bufferwho,
			       
			       fill_len,"",

			       entry->lines, 
			       (entry->lines / 1000   > 0? ""   :	/* spacing the  */
				entry->lines / 100   > 0? " "  :	/* same for the */
				entry->lines / 10  > 0? "  " :	/* lines in ()  */
				"   ")      /*   [wierd]    */
			       );
    
    free_string(&bufferwho);
    free_string(&list_buffer);

    X = 0;
    /* clip_from_string updates X */
    buffer1 = curses_printable_clip(buffer,&X,COLUMNS,
				    &header_without_subj_width,
				    COLUMNS);

    free_string(&buffer);


    /* Max printable width of subject -- note that
       this may be longer than number of characters
       on line -- we can't use string_len as subj_width
       because some characters may consume several positions
       (double width characters)
    */
    subj_width = COLUMNS-header_without_subj_width;

    if (buffer1) {
	menu_Write_to_screen(page,FRM("%S"),buffer1);
	free_string(&buffer1);
    }

    /* complete line with subject. */
    if (subj_width > 0 && entry->subject) {
	struct string * buffer2 = NULL;

	X = 0;
	/* clip_from_string updates X */
	buffer2 = curses_printable_clip(entry->subject,&X,subj_width,
					&was_len,subj_width);

	if (buffer2) {
	    menu_Write_to_screen(page,FRM("%S"),buffer2);
	    free_string(&buffer2);
	}
    } else
	was_len = 0;   /* No subject ? */

    /* If it is highlighted but not with the arrow  cursor,
     * expand subject to fit so that the reverse video bar extends
     * aesthetically the full length of the line.
     */
    if (selected && !arrow_cursor) {
	while (was_len < subj_width) {
	    menu_Writechar(page,' ');
	    was_len++;
	}
    }
        
    if (selected && has_highlighting && ! arrow_cursor) 
	menu_EndXX(page,pg_INVERSE);

    menu_Writechar(page,'\r');
    menu_Writechar(page,'\n');
}

/*
 * Local Variables:
 *  mode:c
 *  c-basic-offset:4
 *  buffer-file-coding-system: iso-8859-1
 * End:
 */
